#
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2009.  All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#
#
# Description:
#
# This script is used to add or strip a codepage information header to the exported .qry, .frm files and so on.   
# The header format is "version = 01;codepage = 936       <CQ_FILE_HEADER_END>" in which "936" is the codepage of the file
# The following are examples of adding the codepage header:
#
# example 1. Add the header to a specified file:
# cqperl cpheader.pl -a c:/defect.frm 1252
#
# example 2. Add the header to all files of a specified directory and its subdirectories:
# ratlperl cpheader.pl -A c:/queries 936
#
# example 3. Add the header to all files that meets the specified regular expression of a specified directory and its subdirectories:
# ratlperl cpheader.pl -a c:/queries .cfg$ 936
#
# The following are examples of striping the codepage header
#
# example 1. Strip the header to a specified file:
# cqperl cpheader.pl -s c:/alldefect.qry
#
# example 2. Strip the header to all files of a specifed directory and its subdirecotories:
# cqperl cpheader.pl -S c:/qrydir 
# 

use CQPerlExt;
use strict;

#Format header string XXX to format "version = 01;codepage = XXX...<CQ_FILE_HEADER_END>"
#"..." means appending several " " to make the length of "XXX..."equals 10 bytes.
sub format_header {
	my $CQ_LATEST_FILE_HEADER_VERSION = "01";
	my $CQ_HEADER_VERSION_PREFIX = "version = ";
	my $CQ_HEADER_CODEPAGE_PREFIX = "codepage = ";
	my $CQ_FILE_HEADER_END = "<CQ_FILE_HEADER_END>\0";

	my ($headerstr) = @_;
	my $leftstr = $CQ_HEADER_VERSION_PREFIX.$CQ_LATEST_FILE_HEADER_VERSION.";".$CQ_HEADER_CODEPAGE_PREFIX ;
	my $len = rindex($headerstr."\$", "\$");
	if ($len <= 10) 
	{
		for (my $i = 10; $i > $len; $i--) 
		{
			$headerstr = $headerstr." ";
		}
		$headerstr = $leftstr.$headerstr.$CQ_FILE_HEADER_END;
	}  
	return $headerstr;
}

#Add specified header to the specified file.
sub add_header_to_file {
  
	my ($filename, $header) = @_;
	my $tmpfilename = "_Temp";
	use File::Copy; 
	copy($filename, $tmpfilename) or die "Copy failed: $!"; 

	open (FILE, ">$filename") or die "$! \n";
	open (TMPFILE, "$tmpfilename") or die "$! \n";   
	binmode(FILE);
	binmode(TMPFILE);
	
	 my $headerinfo = <TMPFILE>;
    
   my $tmp_index = index($headerinfo, "<CQ_FILE_HEADER_END>");
    
    if ($tmp_index >= 0) {
      print "$filename already has codepage header\n";
      seek(TMPFILE, 0 , 0);
    } else {
      seek(TMPFILE, 0 , 0); 
      print FILE "$header" or die "$! \n";
    }
	
	while (<TMPFILE>)
	{
		print FILE "$_";
	}

	close (TMPFILE);
	close (FILE);

	unlink $tmpfilename;
}

#Add specified header to files under the specified directory;
sub add_header_to_dir {
	my ($dir, $expr, $header) = @_;
	opendir(THISDIR, "$dir") or die "$! \n";
	my @allfiles = readdir(THISDIR);

	foreach my $file(@allfiles) {
		if ($file eq "." || $file eq "..")
		{
			next;	
		}
		my $file_ = $dir. "\\" . $file;
		if (-f $file_)
		{
			if (($expr eq "") or ($file_ =~ /$expr/)) 
			{
				add_header_to_file($file_, $header);
			}
		}
		else
		{
			add_header_to_dir($file_, $expr, $header); 
		}
	}
}

#Strip specified header to the specified file.
sub strip_header_to_file {
	my ($filename) = @_;
	my $tmpfilename = "_Temp";
	use File::Copy; 
	copy($filename, $tmpfilename) or die "Copy failed: $!"; 

	open (FILE, ">$filename") or die "$! \n";
	open (TMPFILE, "$tmpfilename") or die "$! \n";

	binmode(FILE);
	binmode(TMPFILE);

	my $tmp_line;
	my $tmp_index = 0;
	my $header_length = 0;

	my $headerinfo = <TMPFILE>;

	$tmp_index = index($headerinfo, "<CQ_FILE_HEADER_END>");

	if ($tmp_index >= 0) {
		$header_length = length("<CQ_FILE_HEADER_END>");

		seek(TMPFILE, $tmp_index + $header_length + 1 , 0); 

	} else {
	  print "$filename has no codepage header.\n";
		seek(TMPFILE, 0 , 0); 
	}

	while (<TMPFILE>)
	{
		print FILE "$_";
	}

	close (TMPFILE);
	close (FILE);

	unlink $tmpfilename;
}

#strip specified header to files under the specified directory;
sub strip_header_to_dir {
	my ($dir, $expr) = @_;
	opendir(THISDIR, "$dir") or die "$! \n";
	my @allfiles = readdir(THISDIR);

	foreach my $file(@allfiles) {
		if ($file eq "." || $file eq "..")
		{
			next;	
		}
		my $file_ = $dir. "\\" . $file;
		if (-f $file_)
		{
			if (($expr eq "") or ($file_ =~ /$expr/)) 
			{
				strip_header_to_file($file_);
			}
		}
		else
		{
			strip_header_to_dir($file_, $expr); 
		}
	}
}

sub Usage {
	print <<EOF;

Usage: cqperl cpheader.pl [-Stripheader|-Addheader] filename|directory [expr] codepage.
Description:

This script is used to add or strip a codepage information header to the exported .qry, .frm files and so on.   
The header format is "version = 01;codepage = 936       <CQ_FILE_HEADER_END>" in which "936" is the codepage of the file
The following are examples of adding the codepage header:

example 1. Add the header to a specified file:
cqperl cpheader.pl -a c:/defect.frm 1252

example 2. Add the header to all files of a specified directory and its subdirectories:
ratlperl cpheader.pl -A c:/queries 936

example 3. Add the header to all files that meets the specified regular expression of a specified directory and its subdirectories:
ratlperl cpheader.pl -a c:/queries .cfg$ 936

The following are examples of striping the codepage header

example 1. Strip the header to a specified file:
cqperl cpheader.pl -s c:/alldefect.qry

example 2. Strip the header to all files of a specifed directory and its subdirecotories:
cqperl cpheader.pl -S c:/qrydir 

EOF
	exit 1;
}


my $argvc = @ARGV;
if ($argvc < 2) 
{
	Usage();
} 
else 
{   
	my ($mode, $path, $expr, $codepage) = ("", "", "", "");

	$mode = $ARGV[0];
	
	if ($mode =~ /^-S/i) {
	  
		if ($argvc == 2)
		{
			($mode , $path) = @ARGV;
		}
		elsif ($argvc == 3)
		{
			($mode, $path, $expr) = @ARGV;
		}
		else
		{
			Usage();
			exit 0;
		}

		print "*********************************************************\n"
			."Starting strip_codepage_header\n"
			."*********************************************************\n";

		if (-f $path) 
		{
			strip_header_to_file($path);
		}
		elsif (-d $path) 
		{
			strip_header_to_dir($path, $expr);
		}
		else 
		{
			print "$path is not a valid directory or file name!";
		}

		print "*********************************************************\n"
			."End strip_codepage_header\n"
			."*********************************************************\n";


	} elsif ($mode =~ /^-A/i) {

		if ($argvc == 3)
		{
			($mode, $path, $codepage) = @ARGV;
		}
		elsif ($argvc == 4)
		{
			($mode, $path, $expr, $codepage) = @ARGV;
		}
		else
		{
			Usage();
			exit 0;
		}

		print "*********************************************************\n"
			."Starting add_codepage_header\n"
			."*********************************************************\n";

		my $header = format_header($codepage);

		if (-f $path) 
		{
			add_header_to_file($path, $header);
		} 
		elsif (-d $path) 
		{
			add_header_to_dir($path, $expr, $header);
		} 
		else 
		{
			print "$path is not a valid directory or file name!";
		}

		print "*********************************************************\n"
			."End add_codepage_header\n"
			."*********************************************************\n";      
	} else {
		Usage();
	}

}
